@@ -4,9 +4,9 @@
 #include "taskflow/algorithm/for_each.hpp"
 #include "taskflow/taskflow.hpp"
 #include <atomic>
-#include <chrono>
 #include <exception>
-#include <numeric>
+#include <stdexcept>
+#include <vector>
 using namespace std::chrono_literals;
 
 template <typename T>
@@ -74,6 +74,9 @@ struct ExtractComp {
         if (m_scorer_flags->flags & RF_SCORER_FLAG_RESULT_F64) {
             return is_first(a, b, m_scorer_flags->optimal_score.f64, m_scorer_flags->worst_score.f64);
         }
+        if (m_scorer_flags->flags & RF_SCORER_FLAG_RESULT_SIZE_T) {
+            return is_first(a, b, m_scorer_flags->optimal_score.sizet, m_scorer_flags->worst_score.sizet);
+        }
         else {
             return is_first(a, b, m_scorer_flags->optimal_score.i64, m_scorer_flags->worst_score.i64);
         }
@@ -150,6 +153,49 @@ struct RF_ScorerWrapper {
     {
         PyErr2RuntimeExn(scorer_func.call.i64(&scorer_func, str, 1, score_cutoff, score_hint, result));
     }
+
+    void call(const RF_String* str, size_t score_cutoff, size_t score_hint, size_t* result) const
+    {
+        PyErr2RuntimeExn(scorer_func.call.sizet(&scorer_func, str, 1, score_cutoff, score_hint, result));
+    }
+};
+
+struct RF_UncachedScorerWrapper {
+    RF_UncachedScorerFunc scorer_func;
+
+    explicit RF_UncachedScorerWrapper(RF_UncachedScorerFunc scorer_func_) : scorer_func(scorer_func_)
+    {}
+
+    RF_UncachedScorerWrapper(const RF_UncachedScorerWrapper&) = delete;
+    RF_UncachedScorerWrapper& operator=(const RF_UncachedScorerWrapper&) = delete;
+
+    RF_UncachedScorerWrapper(RF_UncachedScorerWrapper&& other) : scorer_func(other.scorer_func)
+    {}
+
+    RF_UncachedScorerWrapper& operator=(RF_UncachedScorerWrapper&& other)
+    {
+        if (&other != this) scorer_func = other.scorer_func;
+
+        return *this;
+    };
+
+    void call(const RF_String* str1, const RF_String* str2, const RF_Kwargs* kwargs, double score_cutoff,
+              double score_hint, double* result) const
+    {
+        PyErr2RuntimeExn(scorer_func.call.f64(str1, str2, kwargs, score_cutoff, score_hint, result));
+    }
+
+    void call(const RF_String* str1, const RF_String* str2, const RF_Kwargs* kwargs, int64_t score_cutoff,
+              int64_t score_hint, int64_t* result) const
+    {
+        PyErr2RuntimeExn(scorer_func.call.i64(str1, str2, kwargs, score_cutoff, score_hint, result));
+    }
+
+    void call(const RF_String* str1, const RF_String* str2, const RF_Kwargs* kwargs, size_t score_cutoff,
+              size_t score_hint, size_t* result) const
+    {
+        PyErr2RuntimeExn(scorer_func.call.sizet(str1, str2, kwargs, score_cutoff, score_hint, result));
+    }
 };
 
 template <typename T>
@@ -158,6 +204,9 @@ bool is_lowest_score_worst(const RF_ScorerFlags* scorer_flags)
     if (std::is_same<T, double>::value) {
         return scorer_flags->optimal_score.f64 > scorer_flags->worst_score.f64;
     }
+    else if (std::is_same<T, size_t>::value) {
+        return scorer_flags->optimal_score.sizet > scorer_flags->worst_score.sizet;
+    }
     else {
         return scorer_flags->optimal_score.i64 > scorer_flags->worst_score.i64;
     }
@@ -169,6 +218,9 @@ T get_optimal_score(const RF_ScorerFlags* scorer_flags)
     if (std::is_same<T, double>::value) {
         return (T)scorer_flags->optimal_score.f64;
     }
+    else if (std::is_same<T, size_t>::value) {
+        return (T)scorer_flags->optimal_score.sizet;
+    }
     else {
         return (T)scorer_flags->optimal_score.i64;
     }
@@ -258,6 +310,11 @@ int64_t any_round(int64_t score)
     return score;
 }
 
+size_t any_round(size_t score)
+{
+    return score;
+}
+
 enum class MatrixType {
     UNDEFINED,
     FLOAT32,
@@ -619,3 +676,35 @@ static Matrix cdist_two_lists_impl(const RF_ScorerFlags* scorer_flags, const RF_
 
     return matrix;
 }
+
+template <typename T>
+static Matrix cpdist_cpp_impl(const RF_Kwargs* kwargs, RF_Scorer* scorer,
+                              const std::vector<RF_StringWrapper>& queries,
+                              const std::vector<RF_StringWrapper>& choices, MatrixType dtype, int workers,
+                              T score_cutoff, T score_hint, T score_multiplier, T worst_score)
+{
+    int64_t rows = queries.size();
+    int64_t cols = 1;
+    Matrix matrix(dtype, static_cast<size_t>(rows), static_cast<size_t>(cols));
+
+    if (queries.empty() || choices.empty()) return matrix;
+
+    // TODO: Adjust the batch size based on the query, choices & hardware available
+    const int batchSize = 16;
+
+    run_parallel(workers, rows, batchSize, [&](int64_t row, int64_t row_end) {
+        for (; row < row_end; ++row) {
+            T score;
+            if (choices[row].is_none() || queries[row].is_none())
+                score = worst_score;
+            else
+                RF_UncachedScorerWrapper(scorer->uncached_scorer_func)
+                    .call(&queries[row].string, &choices[row].string, kwargs, score_cutoff, score_hint,
+                          &score);
+
+            matrix.set(row, 0, score * score_multiplier);
+        }
+    });
+
+    return matrix;
+}
